import gulp from 'gulp'
import https from 'https'
import querystring from 'querystring'
import async from 'async'
import fs from 'fs'
import path from 'path'
import shell from 'shelljs'
import fixtures from '../fixtures'

export function getPage(token, url, callback) {
  var opts;
  if (url.indexOf('html') === -1) {
    if (url.charAt(url.length - 1) !== '/') {
      url += '/';
    }
    url += 'index.html';
  }

  opts = {
    host: 'api.thegrid.io',
    port: 443,
    path: '/site/page?' + querystring.stringify({
      url: url
    }),
    method: 'GET',
    headers: {
      'Authorization': 'Bearer ' + token
    }
  };

  return https.get(opts, function(res) {
    var body;
    body = '';
    res.on('data', function(chunk) {
      return body += chunk;
    });
    return res.on('end', function() {
      if (res.statusCode !== 200) {
        return callback(new Error((res.statusCode + ':\n ' + body + ' at ') + opts.path));
      }
      return callback(null, JSON.parse(body));
    });
  });
};

export function updateFixtures(token, baseDir, sites, callback) {

  const updateFixture = function(config, cb) {
    // var siteUrl, targetPath;
    const siteUrl = config.siteUrl;
    const targetPath = path.join(baseDir, config.mock + '.js');
    return getPage(token, siteUrl, function(err, page) {
      if (err) {
        return cb(err);
      }

      var fixtureObj = {
        siteData: page,
        fileName: config.fileName,
        folder: config.folder,
        siteUrl: config.siteUrl,
        parentFolder: config.parentFolder,
      }

      page = 'module.exports = function() { \n return ' + JSON.stringify(fixtureObj, null, 2) + '};';
      return fs.writeFile(targetPath, page, {
        encoding: 'utf8'
      }, function(err) {
        if (err) {
          return cb(err);
        }
        return cb(null, targetPath);
      });
    });
  };

  var sitesConfig = [];

  for (let siteName in sites) {
    var siteObj = sites[siteName];

    sitesConfig.push({
      siteUrl: siteObj.index,
      mock: siteObj.mock,
      fileName: siteObj.fileName,
      folder: siteObj.folder,
    });

    if (sites[siteName].feed) {
      for (let feed of sites[siteName].feed) {
        sitesConfig.push({
          siteUrl: feed.index,
          mock: feed.mock,
          fileName: feed.fileName,
          folder: feed.folder,
        });
      }
    }

    if (sites[siteName].pages) {
      for (let page of sites[siteName].pages) {
        sitesConfig.push({
          siteUrl: page.index,
          mock: page.mock,
          fileName: page.fileName,
          folder: page.folder,
          parentFolder: siteObj.folder,
        });
      }
    }
  }

  //let siteNames = Object.keys(sites);
  return async.map(sitesConfig, updateFixture, callback);
};


export function updateSites(done) {
  var baseDir, fs, token;
  token = 'abcRvxntjTny0UeG1CDTjQzDoZVkePUionjwpDWbZbGvtdfObON1NTnZvlbv3EfYBs2oZEIe';
  baseDir = 'tmp/mocks';
  fs = require('fs');

  if (!fs.existsSync(baseDir)) {
    shell.mkdir('-p', baseDir)
  }

  var allFixtures = fixtures();
  var allFixtureNames = Object.keys(allFixtures);
  var allMockStr = '';
  var allMockNames = [];

  for (let i = 0; i < allFixtureNames.length; i++) {
    allMockStr += 'var ' + allFixtures[allFixtureNames[i]].mock + ' = require ("./' + allFixtures[allFixtureNames[i]].mock + '");\n';
    allMockNames.push(allFixtures[allFixtureNames[i]].mock + ': ' + allFixtures[allFixtureNames[i]].mock);

    if (allFixtures[allFixtureNames[i]].feed) {
      for (let feed of allFixtures[allFixtureNames[i]].feed) {
        allMockStr += 'var ' + feed.mock + ' = require("./' + feed.mock + '");\n';
        allMockNames.push(feed.mock + ': ' + feed.mock);
        console.log(allMockNames)
      }
    }

    if (allFixtures[allFixtureNames[i]].pages) {
      for (let page of allFixtures[allFixtureNames[i]].pages) {
        allMockStr += 'var ' + page.mock + ' = require ("./' + page.mock + '");\n';
        allMockNames.push(page.mock + ': ' + page.mock);
        console.log(allMockNames);
      }
    }
  }
  allMockStr += 'module.exports = {' + allMockNames.join(', ') + '};';

  fs.writeFile(baseDir + '/allMocks.js', allMockStr, {
    encoding: 'utf8'
  }, function(err) {
    if (err) {
      console.log(err);
    }
  });

  return updateFixtures(token, baseDir, allFixtures, (function() {
    return function(err, fixtures) {
      if (err) {
        console.error(err);
      }
      console.log('Updated fixtures: \n', fixtures.join('\n'));
      return done();
    };
  })(this));
};


gulp.task('sites:update', function(done) {
  updateSites(done)
})
