import gulp from 'gulp'
import mocha from 'gulp-mocha'
import selenium from 'selenium-standalone'
import http from 'http'
import connect from 'connect'
import webdriver from 'gulp-webdriver'
import serveStatic from 'serve-static'

const mochaCfg = {
  options: {
    require: [
      'babel-core/register'
    ],
    timeout: 6000,
    reporter: 'spec'
  }
}

let httpServer, seleniumServer;

gulp.task('mochaTest:all', function() {
  return gulp.src(['tests/**/*.spec.js'])
    .pipe(mocha(mochaCfg))
})


gulp.task('e2e:http', (done) => {
  let app = connect().use(serveStatic('tests'));
  httpServer = http.createServer(app).listen(9000, done);
})

gulp.task('e2e:selenium', (done) => {
  selenium.install({logger: console.log}, () => {
    selenium.start((err, child) => {
      if (err) { return done(err); }
      seleniumServer = child;
      done();
    });
  });
})

gulp.task('e2e:start', ['e2e:http', 'e2e:selenium'], () => {
  return gulp.src('wdio.conf.js')
    .pipe(webdriver()).on('error', () => {
      seleniumServer.kill();
      process.exit(1);
    });
});

gulp.task('e2e', ['e2e:start', 'sites:update'], () => {
  httpServer.close();
  seleniumServer.kill();
})
