import webpack from 'webpack'
import path from 'path'
import CopyWebpackPlugin from 'copy-webpack-plugin'
import CleanWebpackPlugin from 'clean-webpack-plugin'

export default {
  cache: true,
  debug: true,
  entry: {
    'ds-renderer': ['webpack-hot-middleware/client?path=/__webpack_hmr','./src/renderer/index']
  },
  output: {
    path: __dirname + '/tmp/dev',
    publicPath: '/',
    filename: '[name].js'
  },
  module: {
    loaders: [
      {
        test: /\.js$/,
        loader: 'react-hot!babel?cacheDirectory',
        exclude: /(node_modules)/
      },
      {
        test: /\.json$/,
        loader: 'json'
      }
    ],
    resolve: {
      extensions: ['', '.js'],
      //modulesDirectories: ['node_modules', 'src'],
      root: [
        path.resolve('../'), path.resolve('../../'), path.resolve('../../../'), path.resolve('/Users/D4/git/the-grid')
      ],
      alias: {
        '@the-grid/multiverse-ui': '/multiverse-ui/src',
        '@the-grid/immutable': '/immutable/src',
        '@the-grid/the-data-modeler': '/the-data-modeler/src',
      }
    }
  },
  plugins: [
    new webpack.optimize.OccurrenceOrderPlugin(),
    new webpack.HotModuleReplacementPlugin({
      multiStep: true
    }),
    new CleanWebpackPlugin(['tmp/dev']),
    new CopyWebpackPlugin([
      { from: './src/renderer/assets', to: 'assets', force: true }
    ])
  ]
}
