import React from 'react'
import Store from './Store'

const KEYS_TO_IGNORE_WHEN_COPYING_PROPERTIES = [
  'arguments',
  'callee',
  'caller',
  'length',
  'name',
  'prototype',
  'type'
];

function cloneComponentProps(source, target) {
  Object.getOwnPropertyNames(source).forEach(key => {
    if (
      KEYS_TO_IGNORE_WHEN_COPYING_PROPERTIES.indexOf(key) < 0 &&
      !target.hasOwnProperty(key)
    ) {
      const descriptor = Object.getOwnPropertyDescriptor(source, key)
      Object.defineProperty(target, key, descriptor)
    }
  });
}

let Contextual = {

  Store,

  make: function (name, {actions, state}) {

    let store = new Store(name, actions, state)

    class ContextualComponent extends React.Component {

      render() {
        return this.props.children
      }

      request(...args) {
        let [storeName] = args

        // catch request
        if (storeName === name) return store.request.apply( store, args.slice(1) )

        if (this.context && this.context.__contextual_names__) {
          if (this.context.__contextual_names__.includes(storeName)) {

            // bubble request
            return this.context.request(...args)
          }
        }

        throw new Error(`Contextual: Component not found for request (${storeName}, ${args[1]}, ...)`)
      }

      getChildContext() {

        let __contextual_names__ = this.context.__contextual_names__

        if (!__contextual_names__) {
          __contextual_names__ = [name]
        }
        else if (!__contextual_names__.includes(name)) {
          __contextual_names__ = [name].concat(__contextual_names__)
        }

        return {
          request: this.request,
          __contextual_names__,
        }
      }

    }

    ContextualComponent.contextTypes = {
      request: React.PropTypes.func,
      __contextual_names__: React.PropTypes.array,
    }

    ContextualComponent.childContextTypes = {
      request: React.PropTypes.func,
      __contextual_names__: React.PropTypes.array,
    }

    // ContextualComponent.propTypes = {
    //   children: React.PropTypes.object.isRequired,
    // }

    return ContextualComponent

  },

  enhance: function (ComponentToEnhance, name, {actions, state}) {


    class Enhanced extends ComponentToEnhance {

      constructor(...args) {
        super(...args)
        this.store = new Store(name, actions, state)
        this.isAwesome = 'boobs'
        this.request = this.request.bind(this)
      }

      request(...args) {
        if (super.request) super.request(...args)


        let [requestedStoreName] = args

        // catch request
        if (requestedStoreName === name) return this.store.request.apply( this.store, args.slice(1) )

        if (this.context && this.context.__contextual_names__) {
          if (this.context.__contextual_names__.includes(requestedStoreName)) {

            // bubble request
            return this.context.request(...args)
          }
        }
        throw new Error(`Contextual: Component not found for request (${requestedStoreName}, ${args[1]}, ...)`)
      }

      getChildContext() {
        const superChildContext = super.getChildContext ?
          super.getChildContext() :
          {}

        let childContext = {...superChildContext}

        let __contextual_names__ = this.context.__contextual_names__

        if (!__contextual_names__) {
          __contextual_names__ = [name]
        }
        else if (!__contextual_names__.includes(name)) {
          __contextual_names__ = [name].concat(__contextual_names__)
        }

        childContext.request = this.request
        childContext.__contextual_names__ = __contextual_names__



        return childContext
      }
    }


    // Class inheritance uses Object.create and because of __proto__ issues
    // with IE <10 any static properties of the superclass aren't inherited and
    // so need to be manually populated.
    // See http://babeljs.io/docs/advanced/caveats/#classes-10-and-below-
    cloneComponentProps(ComponentToEnhance, Enhanced);

    if (process.env.NODE_ENV !== 'production') {
      // This also fixes React Hot Loader by exposing the original components top
      // level prototype methods on the enhanced prototype as discussed in
      // https://github.com/FormidableLabs/radium/issues/219.
      cloneComponentProps(ComponentToEnhance.prototype, Enhanced.prototype);
    }

    //if (Enhancer.propTypes && Enhancer.propTypes.style) {
    //  Enhancer.propTypes = {
    //    ...Enhancer.propTypes,
    //    style: PropTypes.oneOfType([
    //      PropTypes.array,
    //      PropTypes.object
    //    ])
    //  };
    //}

    Enhanced._isContextualEnhanced = true

    Enhanced.displayName =
      ComponentToEnhance.displayName ||
      ComponentToEnhance.name ||
      'ContextualEnhanced';

    Enhanced.contextTypes = {
      ...Enhanced.contextTypes,
      request: React.PropTypes.func,
      __contextual_names__: React.PropTypes.array,
    };

    Enhanced.childContextTypes = {
      ...Enhanced.childContextTypes,
      request: React.PropTypes.func,
      __contextual_names__: React.PropTypes.array,
    };

    return Enhanced

  },

}

export default Contextual