if (typeof window === 'undefined') {
  //Required for ReactDOMServer. Needs to be done
  //before importing React package otherwise Babel fails
  //since Babel execute the code.
  console.log('Generating virtual DOM with jsdom.');
  require('jsdom-global')();
  console.log('window should be available: ');
}

import plumber from './plumber';
import componentRegistry from '../renderer/componentRegistry';

const DS_NAME = 'ds-d4';

let dsD4Config = {
  $name: [DS_NAME],
  $formality: 0.3,
  $formalityWeight: 1,

  itemsGeneration: {header: true, navigation: true, footer: true},

  'the-curator': {
    curateOnItems: true,
    curateOnGrid: false,
    harmonies: {
      index: [
        ($) => {

          $('v-bind', [ $('or',['item:dont-cover-crop', 'item:dont-cover-overlay']), 'DeviceLaptop' ] )

          $('markov', {
            branchName: 'COMPONENT',
            legend: [

              'MultiCtxOnSubGrid',
              'DeviceLaptop', 'MonoBasic', 'MultiBasic', 'Fallback',

              'NavHamburger', 'SiteHeader', 'Cta',

              'LoadMoreWButton', 'Footer3col',
            ],
            transitions: [
              {
                vector: [
                  0.1,
                  0.1, 0.1, 0.1, 0.1,
                  100000, 0.1, 0.1,
                  0.1, 0.1
                ],
                at: 0,
              },
              {
                vector: [
                  0.1,
                  0.1, 0.1, 0.1, 0.1,
                  0.1, 50000, 50000,
                  0.1, 0.1
                ],
                at: 1,
              },
              {
                vector: [
                  0.1,
                  0.1, 0.1, 0.1, 0.1,
                  0.1, 0.1, 0.1,
                  0.1, 100000
                ],
                at: -1,
              },
              {
                vector: [
                  10000000000000000000000000,
                  1000, 1000, 1000, .1,
                  0.1, 0.1, 0.1,
                  0.1, 0.1,
                ],
              },
            ],
          });
        },
      ],
      permalink: [
        ($) => {
          $('markov', {
            branchName: 'COMPONENT',
            legend: [
              'NavHamburger',
              'ArticleSiteHeader',
              'ArticleFallback',
              'Fallback',
              'Footer3col'
            ],
            transitions: [
              {
                vector: [100000, 0.1, 0.1, 0.1, 0.1],
                at: 0,
              },
              {
                vector: [0.1, 100000, 0.1, 0.1, 0.1],
                at: 1,
              },
              {
                vector: [0.1, 0.1, 0.1, 0.1, 100000],
                at: -1,
              },
              {
                vector: [0.1, 0.1, 10000, 0.1, 0.1],
              },
            ],
          });
        },
      ],
    },
  },

  'the-painters': {
    index: {
      'ui-components': componentRegistry.allComponents,
    },
    permalink: {
      'ui-components': componentRegistry.permalinkComponents,
    }
  },

  'the-stylist': {
    'the-typographer': {
      generateCSS: false,
      generateLoadFontScript: false,
    },
    'the-beautician': {

      siteBeautyverse: {
        'site:dec-device': {
          '$Y_site:dec-device': {
            'no'  : { $class: ['site:dec-device--no'],   $formality: 0 },
            'can' : { $class: ['site:dec-device--can'],  $formality: .005 },
          },
        },
        'site:dec-inset-bg': {
          '$Y_site:dec-inset-bg': {
            'no'    : { $class: ['dec-inset-bg--no']  , $formality: .000},
            'tint'  : { $class: ['dec-inset-bg--tint']  , $formality: .002},
            'blur'  : { $class: ['dec-inset-bg--blur']  , $formality: .004},
            'gmap'  : { $class: ['dec-inset-bg--gmap']  , $formality: .006},
            'slash' : { $class: ['dec-inset-bg--slash'] , $formality: .008},
          },
        },
      },
      siteHarmonies: [
        ($) => {

        },
      ],

      sectionsBeautyverse: {
        //'dec-device': {
        //  'Y_dec-device': {
        //    'auto'      : { $class: ['dec-device--auto dec-inset-bg--yes'] },
        //    'no'        : { $class: ['dec-device--no'] },
        //  },
        //},
      },
      sectionsHarmonies: [
        ($) => {
          //$('v-if-then', 'site:dec-device--no', 'dec-device--no')
          //$('v-if-then', $('not',['section:dec-device--can']), 'dec-device--no')
          //$('v-if-then', $('not',['item:dec-device--can']), 'dec-device--no')


          //$('markov', {
          //  branchName: 'dec-device',
          //  legend: ['auto', 'no'],
          //  transitions: [
          //    {
          //      vector: [1, .00000000001],
          //    },
          //  ],
          //});
        }
      ],
    },

    'the-colorist': {
      siteHarmonies: [],
      sectionsHarmonies: [],
    },

    'the-lensman': {
      harmonies: [
        ($) => {
          $('on', 'passthrough');
        },
      ],
    },

    'the-style-solver': [
      {
        contractName: 'svg-filter',
        site: {
          filter: {
            Y_filter: {
              'gradientmap': { filter: 'gradientmap', $class: ['site:gradientmap'] },
              'blur': { filter: 'blur', $class: ['site:blur'] },
            }
          }
        },
        siteHarmonies: [],
        sections: {
          filter: {
            Y_filter: {
              'gradientmap': { filter: 'gradientmap', $class: ['gradientmap'] },
              'blur': { filter: 'blur', $class: ['blur'] },
            },
          },
        },
        sectionsHarmonies: [
          ($) => {
            $('v-if-then', 'colors-light', 'blur');
            $('v-if-then', 'colors-dark', 'gradientmap');
          }
        ],
      },
      {
        contractName: 'animation',
        sections: {
          filter: {
            Y_filter: {
              'spring': { filter: 'spring', $class: ['spring'] },
              'linear': { filter: 'linear', $class: ['linear'] },
            },
          },
        },
        sectionsHarmonies: [
          ($) => {
            $('v-if-then', 'blur', 'linear');
            $('v-if-then', 'gradientmap', 'spring');
          }
        ],
      },
    ]
  },

  'the-plumber': {
    pageBuilder: plumber,
  },
};

export default dsD4Config;
