import gom from '../../../node_modules/gom/browser/gom.js';
import packgjson from '../../../package.json';
import base64 from '../helpers/base64';

let generateGomHtml = function(appData, reactHtml, grid, siteData, headContent = '', scripts = '') {
  let $ = gom();
  let siteConfig = siteData.get('config').toJS();
  let componentUrl;

  if (process.env.DEV && process.env.DEV === '1') {
    componentUrl = '/ds-renderer.js';
  } else {
    componentUrl = `https://cdn.thegrid.io/design-systems/the-composer/ds-d4/v${packgjson.version}/ds-renderer.js`;
  }

  let page = $('html', {dir: 'ltr', lang: 'en'}, [
    $('head', null, [
      $('title', null, siteData.get('title')),
      headContent,
      //$('style', {id: 'web-font-binder', type: 'text/css'}, grid['the-stylist']['the-typographer'].htmlStyleTagContent),
      ...generateFavicon($, siteConfig),
      ...generateAnalytics($, siteConfig),
      ...generateOpengraphMetas($, siteData.toJS()),
      ...generateSiteRss($, siteConfig),
    ]),
    $('body', null, [
      $('div', {id: 'app'}, reactHtml), //$('div', {id: 'app'}, reactHtml), //$('div', {id: 'app'}),
      scripts,
      $('script', {id: 'pageReactInitState'}, `window.pageReactInitState = '${base64.encode(JSON.stringify(appData))}'`),
      $('script', {id: 'renderer', src: `${componentUrl}`}),
      //$('script', {id: 'web-font-loader'}, grid['the-stylist']['the-typographer'].htmlScriptTagContent),
    ]),
  ]);

  let doctype = `<!DOCTYPE html>\n`;
  let html = doctype + $.render(page);
  return html;
};

function generateFavicon($, siteConfig) {
  let arr = [];
  if (siteConfig.favicon) arr.push($('link', {rel: 'icon', href: siteConfig.favicon}));
  return arr;
}

function generateAnalytics($, siteConfig) {
  let tags = [];

  // TODO: verify how all these tags should be constructed

  if (siteConfig.analytics && siteConfig.analytics.google) {
    let script = `
    (function(i,s,o,g,r,a,m) {i['GoogleAnalyticsObject']=r;i[r]=i[r]||function() {
    (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
    m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
    })(window,document,'script','https://www.google-analytics.com/analytics.js','ga');
    ga('create', '${siteConfig.analytics.google}', 'auto');
    ga('send', 'pageview');
    `;
    tags.push($('script', {type: 'text/javascript', role: 'google-analytics'}, script));
  }

  return tags;
}

function generateOpengraphMetas($, siteData) {
  let siteConfig = siteData._config;
  // no appId no glory?
  if (!siteConfig.opengraph || !siteConfig.opengraph.appId) return [];

  // TODO: is this opengraph stuff correct?
  let metas = [
    $('meta', {property: 'og:site_name', content: 'the-grid'}),
    $('meta', {property: 'og:url', content: '//' + siteData.siteUrl}),
    $('meta', {property: 'og:appId', content: siteConfig.opengraph.appId}),
  ];

  if (siteConfig.opengraph.image) {
    metas.push($('meta', {property: 'og:image', content: siteConfig.opengraph.image}));
  }

  if (siteConfig.opengraph.type) {
    metas.push($('meta', {property: 'og:type', content: siteConfig.opengraph.type}));
  }

  if (siteConfig.opengraph.locale) {
    metas.push($('meta', {property: 'og:locale', content: siteConfig.opengraph.locale}));
  }

  return metas;
}

function generateSiteRss($, siteConfig) {
  let arr = [];
  if (siteConfig.collections && siteConfig.collections.some((obj) => {
    if (obj.feed) {
      // TODO: probably want to refine this bit
      arr.push($('link', {rel: 'alternate', type: 'application/rss+xml', title: 'RSS', href: '.' + obj.feed.path}));
      // TODO: could a site have multiple rss feeds (technically possible) to list here?
      return true;
    }
  }));
  return arr;
}

export default generateGomHtml;
