import componentRegistry from '../../renderer/componentRegistry'

let removeSolvingOnlyItemData = function(item) {
  for (let block of item.content) {
    if (block.cover && block.cover.histogram) delete block.cover.histogram;
    if (block.cover && block.cover.saliency) {
      if (block.cover.saliency.polygon) {
        delete block.cover.saliency.polygon;
      }
      if (block.cover.saliency.bbox) delete block.cover.saliency.bbox;
      if (block.cover.saliency.regions) delete block.cover.saliency.regions;
    }
  }
  return item;
};

let removeSolvingOnlySiteData = function(siteData) {
  delete siteData._config.image_filters;
  delete siteData.items;
  delete siteData._items;
  return siteData;
}

let prepareAppData = function(grid, siteData = {}, options = {}) {
  let {sections} = grid;
  let {globalComponentsKey} = options;

  let jsonSiteData = removeSolvingOnlySiteData(siteData.toJS());

  // let sections = grid.sections;
  let appData = {
    globalComponentsKey: globalComponentsKey,
    items: {},
    sections: [],
    contracts: grid['the-stylist'],
    siteData: jsonSiteData,
  };

  for (let i = 0; i < sections.length; i++) {
    let section = sections[i];
    let compId = section.uiComponent.componentverse.id;

    let initSectionData = {
      componentId: compId,
      itemsToRenderIds: [],
      contracts: section['the-stylist'],
    };

    let compverse = componentRegistry[compId].componentverse;
    if (compverse.getCompData && typeof compverse.getCompData === 'function') {
      initSectionData.compData = compverse.getCompData(
        section.itemsToRender,
        section['the-stylist'],
        grid['the-stylist'],
        siteData);
    }

    let {itemsToRenderIds} = initSectionData;

    let itemsToRender = section.itemsToRender;

    for (let i = 0; i < itemsToRender.length; i++) {
      let item = itemsToRender[i];

      let jsonItem = item.toJS();
      if (!appData.items[jsonItem.id]) {
        appData.items[jsonItem.id] = removeSolvingOnlyItemData(jsonItem);
      }
      itemsToRenderIds.push(jsonItem.id);
    }

    if (itemsToRender[0].get('isNavigation')) {
      appData.navigation = initSectionData;
    } else if (itemsToRender[0].get('isFooter')) {
      appData.footer = initSectionData;
    } else {
      appData.sections.push(initSectionData);
    }
  }

  //add a section index used by some component to know if there are first in the children
  for (let sectionIndex = 0; sectionIndex < appData.sections.length; sectionIndex++) {
    let section = appData.sections[sectionIndex];
    section.sectionIndex = sectionIndex;
  }

  return appData;
};

export default prepareAppData;
