import React from 'react';
import { Link } from 'react-router'

// You can pass an item as props which will extract the URL for it.
// If page is permalink and access directly, use a href to return to root app.
class LinkBlock extends React.Component {

  render() {
    let href;
    let font = this.props.font ? this.props.font : '';
    let styles = this.props.styles ? this.props.styles : '';
    let linkClasses = font + ' ' + styles;

    if (this.props.item) {
      let item = this.props.item;
      href = item.get('url');

    } else if (this.props.href || this.props.url) {
      href = this.props.href ? this.props.href : this.props.url;

    } else if (this.props.block) {
      //get the first link in the html of the block
      let block = this.props.block;
      let html = block.get('html');
      if (html && html.indexOf("http") !== -1) {
        href = html.substring(html.indexOf('http'),html.indexOf('">'))
      }
    }

    if (href) {
      if (typeof window !== 'undefined' && window.theGridSiteType === 'permalink') {
        return (
          //HTTP GET back to root app.
          <a href={`../${href}`} className={linkClasses}>{this.props.children}</a>
        );
      } else if (href.indexOf('http') === 0 || href.indexOf('tel:') === 0 || href.indexOf('mailto:') === 0) {
        return (
          <a href={href} className={linkClasses}>{this.props.children}</a>
        );
      } else {
        // site internal link and not permalink so we can use react-router.
        let to = href.substring(0, href.indexOf('/'));
        if (!to) to = href;

        return (
          <Link className={linkClasses} to={to}>{this.props.children}</Link>
        );
      }
    } else {
      console.log('no link found');
      return (
        <span className={linkClasses}>{this.props.children}</span>
      );
    }
  }
}

export default LinkBlock;
