import React from 'react'
import {Component} from '@the-grid/multiverse-ui'
import CoverMeta from '../cover-meta'

let displayName = 'Cover'

class Cover extends Component {

  render() {
    let {
      struct: cover,
      children
    } = this.props

    if (!cover) return null

    let focalPoint = cover.get('focalPoint') || ['50%', '50%']
    let sources = cover.get('pictureImgfloUrls')
    let sourcesArray = []

    let metaStruct = cover.get('metadata')

    if (sources) {
      for (let i = 0, len = sources.length; i < len; i++) {
        sourcesArray.push(<source
          key={i}
          media={`(min-width: ${sources[i].width}px)`}
          srcSet={sources[i].src}
            style={{
              objectPosition: `${focalPoint[0]} ${focalPoint[1]}`
            }}
          />);
      }
    }


    let className = 'cover'
    className += ' orientation--' + cover.get('orientation')

    return (

      <figure {...this.elProps({ className })}>

        <picture>
          { sourcesArray }
          <img
            data-width={cover.get('width')}
            data-height={cover.get('height')}
            data-ratio={cover.get('width')/cover.get('height')}
            src={cover.get('src')}
            style={{
              objectPosition: `${focalPoint[0]} ${focalPoint[1]}`
            }}
          />
        </picture>

        { !metaStruct ? null : (
          <CoverMeta struct={metaStruct} />
        )}

        { children || null }

      </figure>
    )
  }
}

Cover.displayName = displayName

Cover.propTypes = {
  struct: React.PropTypes.any,
  children: React.PropTypes.any,
}

export default Cover