import {React, Component, Styleverse} from '../../../deps'
let {PropTypes} = React

let Hookable = {

  make: function (name, o) {

    let {
      tag,
      getTag,
      tagProps,
      className,
      defaultProps,
      init,
      willMount,
      didMount,
      WillUnmount,
      willRender,
      styles,
      background,
      layout,
      styleverseProps,
      contextTypes,
      propTypes,
      displayName,
    } = o

    if (!propTypes) propTypes = {}
    if (!className) className = ''
    if (!styleverseProps) styleverseProps = {}
    if (!layout || typeof layout !== 'function') throw new Error("Hookable make(" + name + " , {layout}) must be a function")

    class HookedComponent extends Component {

      constructor(...args) {
        super(...args)
        if (init) init.call(this)
      }

      componentWillMount() {
        if (willMount) willMount.call(this, this.props)
        if (willRender) willRender.call(this, this.props)
      }

      componentDidMount() {
        if (didMount) didMount.call(this)
      }

      componentWillUnmount() {
        if (willUnmount) willUnmount.call(this)
      }

      componentWillUpdate(nextProps) {
        if (willRender) willRender.call(this, nextProps)
      }

      render() {
        let _tag = (!tag && getTag) ? getTag.call(this) : tag
        if (_tag === null) return null
        if (!_tag) tag = 'div'

        let elProps = this.elProps({
          className: (typeof className === 'function') ? className.call(this) : className
        })
        if (tagProps) {
          elProps = Object.assign( {}, elProps, (typeof tagProps === 'function') ? tagProps.call(this) : tagProps )
        }
        let {id} = elProps
        this.id = id
        this.scope = '#' + id
        let children = this.renderStyleverse( this.renderChildren() )
        if (!(children instanceof Array)) return React.createElement(_tag, elProps, children)
        return React.createElement(_tag, elProps, ...children)
      }

      renderStyleverse(children) {
        let _styleverseProps = (typeof styleverseProps === 'function') ? styleverseProps.call(this) : styleverseProps
        // WARNING DONT MUTATE!
        if (this.props.styleverseProps) _styleverseProps = this.props.styleverseProps.call(this, _styleverseProps)

        if (!Object.keys(_styleverseProps).length) return children
        return React.createElement(Styleverse, _styleverseProps, ...children)
      }

      renderChildren() {
        return [
          this.renderStyles(),
          this.renderBackground(),
          this.renderLayout()
        ]
      }

      renderStyles() {
        if (typeof styles === 'function') return styles.call(this)
        if (styles) return styles
        return null
      }

      renderBackground() {
        let id = this.id
        let bg = <div id={id+'-bg'} className='bg'>{this.props.renderInBackground && this.props.renderInBackground.call(this)}</div>
        if (!this.props.wrapBackground) return bg
        return this.props.wrapBackground.call(this, bg)
      }

      renderLayout() {
        let renderedLayout = layout.call(this)
        if (this.props.wrapLayout) return this.props.wrapLayout.call(this, renderedLayout)
        return renderedLayout
      }

    }

    HookedComponent.propTypes = {
      styleverseProps: PropTypes.func, // WARNING DONT MUTATE!
      renderInBackground: PropTypes.func, // TODO, some other API
      wrapBackground: PropTypes.func,
      wrapLayout: PropTypes.func,
      ...propTypes
    }

    if (displayName) HookedComponent.displayName = displayName
    if (contextTypes) HookedComponent.contextTypes = contextTypes


    class HookableMount extends React.Component {

      componentWillMount() {
        let that = this
        this.context.request('renderer', name, (rendererHook) => {
          that.rendererHook = rendererHook
        })
      }

      render() {
        if (!this.rendererHook) return React.createElement(HookedComponent, this.props)
        let {component: View, props} = this.rendererHook({component: HookedComponent, props: this.props})
        return React.createElement(View, props)
      }

    }

    HookableMount.contextTypes = {
      request: PropTypes.func.isRequired,
    }

    return HookableMount

  }
}


export default Hookable