import {React, Device, Map, Style} from '../../../../../deps'

let unitClassifier = function (unit) {

  // no text
  if (unit.get('text')) {
    return
  }

  let cover = unit.get('cover')
  if (!cover) return
  if (cover.get('width') < 325) return

  let className = 'dec-device'
  if (cover.get('orientation') === 'landscape') {
    className += ' dec-device--macbook'
  }
  else if (cover.get('orientation') === 'portrait') {
    className += ' dec-device--iphone'
  }

  return className
}

let $classToObj = function ($class) {

  let parts = $class.split(' ')
  let obj = {}
  parts.forEach(function (part) {
    if (!part.includes('--')) return
    let [key, val] = part.trim().split('--')
    obj[key] = val
  })
  return obj

}


let plugin = Map({

  name: 'dec-device',

  shouldPlugin: function ($class) {
    if (!$class) return false
    return $class.includes('dec-device') && !$class.includes('dec-device--no')
  },

  unitClassifier,

  'content-unit': function ({component, props}) {
    let {struct} = props
    if (!struct) return {component, props}
    let $class = unitClassifier(struct)
    if (!$class) return {component, props}
    let _wrapLayout = props.wrapLayout
    props = Object.assign({}, props)
    props.className = props.className ? props.className : ''
    props.className += ' ' + $class
    props.wrapLayout = function (children) {
      if (_wrapLayout) children = _wrapLayout.call(this, children)
      return (
        <span>
        <Style perf={true} scope={this.scope}>
        {`

            :scope {
              .device {
                pos -- a
                w -- initial
                h -- initial
                t -- 0
                r -- 0
                l -- 0
                b -- 0
              }
              //&.dec-device--macbook {
              //  .device {
              //    t -- -8vh
              //    b -- -8vh
              //    l -- -8vh
              //  }
              //}
              &.dec-device--iphone {
                bg-clr -- chroma-box-pop
                .device {
                  t -- 1goldy
                  b -- 1goldy
                }
                img {
                  object-position -- 50% 0% !important
                  object-fit -- contain !important
                  bg-clr -- chroma-box
                }
              }
            }
        `}
        </Style>
        <Device className={'device ' + $class} type={$classToObj($class)['dec-device']}>
          {children}
        </Device>
        </span>
      )
    }
    return {component, props}
  }
})


export default plugin