import {React, Component, Styleverse, Style} from '../../../deps';
import {AttributionBlock, blockMap} from '../../blocks';
import ContentUnit from '../../components/content-unit'
import Hookable from '../../components/hookable'
import style from './style'

let displayName = 'MultiSection'

let Section = Hookable.make('section', {

  tag: 'section',

  className: function () {
    let className = 'section--multi'
    let {contracts} = this.props
    let itemsClasses = contracts.itemsClasses
    let colorClasses = contracts['the-colorist'].$class
    if (itemsClasses) className += ' ' + itemsClasses.join(' ')
    if (colorClasses) className += ' ' + colorClasses.join(' ')
    return className.replace(/:/g,'_')
  },

  init: function () {

  },

  // willMount: function () {
  //   this.item = this.props.items
  // },

  styles: function () {
    let scope = this.scope
    return style(scope, this.props.stylesheet)
  },

  layout: function () {
    let {
      units,
      item,
    } = this.props
    let id = this.id
    return (
      <div id={id+'-unit-layout'} className='layout'>
        {units.map((unit,i)=>{
          return <ContentUnit key={"unit-"+i} struct={unit} item={item} />
        })}
      </div>
    )
  },

  styleverseProps: function () {

    let {
      contracts
    } = this.props

    let chromaLocality = contracts['the-colorist']['colorverse']['colors-locality']['locality']
    let chromaType = contracts['the-colorist']['colorverse']['colors-solution']['color']

    // TODO
    if (chromaType === 'white') chromaType='light'
    if (chromaType === 'black') chromaType='dark'
    let chromaProps = {type: chromaType}
    if (chromaLocality === 'local') {
      chromaProps.colors = contracts['the-colorist']['colorsSolution']['palette'].map((c) => `rgb(${c.r}, ${c.g}, ${c.b})`)
    }

    let styleverseProps = {
      breakpoint: {
        target: 'window',
      },
      layout: {},
      typography: {},
      chroma: chromaProps,
    }

    return styleverseProps
  },

})

Section.propTypes = {
  units: React.PropTypes.object.isRequired,
  sectionId: React.PropTypes.string,
  contracts: React.PropTypes.object,
  siteData: React.PropTypes.object,
  itemsToRender: React.PropTypes.array,
}

Section.displayName = displayName

export default Section
