import React from 'react';
import Component from '../base-component';
import componentRegistry from '../componentRegistry';
import {ItemStruct, SiteDataStruct} from '@the-grid/the-data-modeler';
import ViewManager from './view-manager'
import {Styleverse, Style} from '@the-grid/multiverse-ui'
import Styles from '../components/styles'
import SiteContext from '../components/site-context'
import RendererContext from '../components/renderer-context'

class ViewContainer extends Component {
  constructor(...args) {
    super(...args);
    this.siteData = null;
  }

  componentWillMount() {
    let routeProps = this.props.route;
    let initStateItems = routeProps.initState.items;
    let siteContracts = routeProps.initState.contracts;

    if (!this.siteData) {
      let items = [];
      let itemKeys = Object.keys(initStateItems);
      for (let i = 0; i < itemKeys.length; i++) {
        items.push(initStateItems[itemKeys[i]]);
      }
      let siteData = routeProps.initState.siteData;
      siteData.items = items;
      this.siteData = new SiteDataStruct(siteData);
    }

    let getReactComponent = (section) => {
      let componentToRender = componentRegistry[section.componentId];
      let itemsToRender = {};

      if (section.itemsToRenderIds) {
        itemsToRender = section.itemsToRenderIds.map((itemId) => {
          return new ItemStruct(initStateItems[itemId]);
        })
      }

      let contracts = section.contracts;
      return React.createElement(
        componentToRender,
        {
          itemsToRender,
          contracts,
          siteContracts,
          siteData: this.siteData,
        }
      );
    };

    let navInitState = routeProps.initState.navigation;
    let navigation = null;
    if (navInitState) {
      navigation = getReactComponent(navInitState);
    } else {
      console.log('No navigation for this site!');
    }

    let footerInitState = routeProps.initState.footer;
    let footer = null;
    if (footerInitState) {
      footer = getReactComponent(footerInitState);
    } else {
      console.log('No footer for this site!');
    }

    this.setState({navigation, footer, siteContracts});
  }

  render() {

    let brandColors = this.siteData.get('config.color.brandColors').toJS()


      // barker's bites
      // JJ wines
       //"brandColors": [
       //  "#FFFFFF",
       //  "#f3d185",
       //  "#070e1e",
       //  "#000000",
       //  "#82621b"
       //],

         // "brandColors": [
//            "#FFFFFF",
//            "#070e1e",
//            "#c31c5f",
//            "#000000",
//            "#6cccc8"
//          ],

    brandColors =  [
      "#f9f7f4",
      "#070e1e",
      "#fc739e",
      "#0dd0b3",
      "#197fa6"
    ]


    return (
      <SiteContext site={this.siteData}>
      <RendererContext $class="">
      <Styleverse id="app-root" perf={   false}
        base={{renderCSS: false}}
        breakpoint={{target: 'window'}}
        chroma={{renderCSS: false, colors: brandColors, type: 'light', typeIndex: 0}}
        typography={{renderCSS: false}}
        layout={{renderCSS: false}}
      >
        <Styles scope="html">
          <Style perf={true}>{`

            * {
              box-szg -- border-box
              pad -- 0
              mar -- 0
            }

            html {
              bg-clr -- chroma-bg
            }

            h1, h2, h3, h4, h5, h6, p, picture, iframe, quote, ol, ul {
              mar-b -- 1.5rem
              max-w -- 100%
            }

            button {
              bdr -- n
            }

            .author > * {
              whs -- nowrap
            }

            #wrapper {
              ovf-x -- hidden
              w -- 100%
            }

            ::selection {
              bg-color -- chroma-selection
              color -- chroma-selection-text
              text-shadow -- none
            }

          `}</Style>
          <style
            dangerouslySetInnerHTML={{__html: `
            html body figure {
              line-height: 0;
              -webkit-margin-before: 0;
              -webkit-margin-after: 0;
              -webkit-margin-start: 0;
              -webkit-margin-end: 0;
            }
            `}} />
        </Styles>
        <div id='wrapper'>
          {this.state.navigation}
          <ViewManager {...this.props} siteData={this.siteData} />
          {this.state.footer}
        </div>
      </Styleverse>
      </RendererContext>
      </SiteContext>
    );
  }
}

ViewContainer.propTypes = {
  children: React.PropTypes.object,
  route: React.PropTypes.object,
};

export default ViewContainer;
