import React from 'react';
import Component from '../base-component';
import componentRegistry from '../componentRegistry';
import {ItemStruct} from '@the-grid/the-data-modeler';
import base64 from '../../ds/helpers/base64';

let sectionIndex = -1;
class ViewManager extends Component {
  constructor(...args) {
    super(...args)
    this.stateCache = {}
    this.httpRequest = null
    this.loadInit = true
  }

  componentWillMount() {
    if (this.props.route.initState) {
      this.loadComponentsFromInitState(this.props);

      if (this.loadInit && !this.props.route.serverSide) {
        let links = this.props.siteData.get('links')
        if (links && links.size) {
          let nextPageLink
          links.forEach((link) => {
            if (link.get('rel') !== 'next') return false
            nextPageLink = link.get('href')
          })
          if (nextPageLink) {
            setTimeout(() => {
              this.ajaxLoadContent(nextPageLink, true);
            }, 2000)
            this.loadInit = false
          }
        }
      }
    } else {
      this.ajaxLoadContent(this.props.location.pathname);
      this.setState({sections: null, currentPath: this.props.location.pathname});
    }
  }

  componentDidMount() {
  }

  componentWillReceiveProps(nextProps) {
    if (this.state.currentPath !== nextProps.location.pathname) {
      this.stateCache[this.state.currentPath] = this.state;

      if (this.stateCache[nextProps.location.pathname]) {
        this.setState(this.stateCache[nextProps.location.pathname]);
      } else {
        let loadingComp = React.createElement(componentRegistry.LoadingDefault, {key: 'loading'});
        this.ajaxLoadContent(nextProps.location.pathname);
        this.setState({sections: [loadingComp], currentPath: nextProps.location.pathname});
      }
    }
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (this.state.currentPath === nextProps.location.pathname &&
        this.state.sections === nextState.sections) {
      return false;
    }
    return true;
  }

  render() {
    return (
      <div className="section-layout layout">
        {this.state.sections}
      </div>
    );
  }

  setupSectionsReactComponents(initStateSections, initStateItems) {
    return initStateSections.map((section) => {
      let componentToRender = componentRegistry[section.componentId];
      let itemsToRender;

      let sectionItemId = null
      if (section.itemsToRenderIds) {
        itemsToRender = section.itemsToRenderIds.map((itemId) => {
          if(!sectionItemId) {
            sectionItemId = itemId
          }
          return new ItemStruct(initStateItems[itemId]);
        })
      }
      let contracts = section.contracts;

      sectionIndex++;
      let sectionId = 'section-' + section.componentId + '-' + sectionItemId;
      return React.createElement(
        componentToRender,
        {
          itemsToRender,
          contracts,
          sectionId,
          siteData: this.props.siteData,
          addSections: this.onAddSections.bind(this),
          sectionIndex: sectionIndex,
          key: sectionId,
          siteContracts: this.props.route.initState.contracts,
        }
      );
    });
  }

  onAddSections(pathname) {
    this.ajaxLoadContent(pathname.get('href'), true);
  }

  //fixme: https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API much better here
  ajaxLoadContent(pathname, addToCurrentSections = false) {
    let url;
    if (pathname.indexOf('.html') === -1) {
      if (pathname.charAt(0) === '/') pathname = pathname.slice(1);
      if (pathname.indexOf('/') === pathname.length - 1) pathname = pathname.slice(0, -1);
      url = pathname + '/index.html';
    } else {
      url = pathname;
    }

    this.httpRequest = new XMLHttpRequest();
    this.httpRequest.onreadystatechange = this.onReceiveAjaxContent.bind(this, addToCurrentSections, pathname);
    this.httpRequest.open('GET', url);
    this.httpRequest.send();
  }

  onReceiveAjaxContent(addToCurrentSections, pathname) {
    let httpRequest = this.httpRequest;
    if (httpRequest.readyState === XMLHttpRequest.DONE) {
      if (httpRequest.status === 200) {
        let regex = new RegExp('<script id="pageReactInitState">(.*?)</script>');
        let scriptContent = regex.exec(httpRequest.responseText);
        let initStateScript = scriptContent[1];

        initStateScript = initStateScript.substring("window.pageReactInitState = '".length, initStateScript.length - 1);
        initStateScript = base64.decode(initStateScript);
        initStateScript = JSON.parse(initStateScript);

        if (addToCurrentSections) {
          let newSections = this.setupSectionsReactComponents(initStateScript.sections, initStateScript.items, this.props.route);
          let currentSection = this.state.sections;
          currentSection.pop();
          let that = this;
          that.setState({sections: currentSection.concat(newSections), currentPath: that.props.location.pathname});
        } else {
          let sections = this.setupSectionsReactComponents(initStateScript.sections, initStateScript.items, this.props.route);
          this.setState({sections, currentPath: this.props.location.pathname});
        }
        this.PubSub.publish('onSectionsLoaded', {pathname});
      } else {
        let missingPageComp = React.createElement(componentRegistry.MissingPageDefault);
        if (!addToCurrentSections) {
          this.setState({sections: [missingPageComp], currentPath: this.props.location.pathname});
        }
        this.PubSub.publish('onSectionsLoaded', {err: httpRequest});
      }
    }
  }

  loadComponentsFromInitState(props) {
    let initStateSections = props.route.initState.sections;
    let initStateItems = props.route.initState.items;
    let sections = this.setupSectionsReactComponents(initStateSections, initStateItems, props.route);
    this.setState({sections, currentPath: props.location.pathname});
  }

}

ViewManager.propTypes = {
  route: React.PropTypes.object,
  location: React.PropTypes.object,
}

export default ViewManager;
